@@ -41,7 +41,7 @@
 
 #include <sys/queue.h>
 #include <sys/tree.h>
-#include "vis.h"
+#include "bsd-vis.h"
 #include "xmalloc.h"
 
 #ifdef HAVE_SYS_TIME_H
@@ -141,6 +141,10 @@ void arc4random_buf(void *, size_t);
 uint32_t arc4random_uniform(uint32_t);
 #endif
 
+#if !defined(SSL_OP_NO_CLIENT_RENEGOTIATION) && !defined(LIBRESSL_VERSION_NUMBER)
+#define SSL_OP_NO_CLIENT_RENEGOTIATION 0
+#endif
+
 #ifndef HAVE_ASPRINTF
 int asprintf(char **, const char *, ...);
 #endif 
@@ -218,6 +222,7 @@ void *recallocarray(void *, size_t, size_t, size_t);
 #endif
 
 #ifndef HAVE_ERRC
+__attribute__ ((noreturn))
 void errc(int, int, const char *, ...);
 #endif
 
@@ -229,6 +234,14 @@ int inet_net_pton(int, const char *, void *, size_t);
 #define pledge(promises, paths) 0
 #endif
 
+#ifndef HAVE_MALLOC_CONCEAL
+#define malloc_conceal malloc
+#endif
+
+#ifndef HAVE_CALLOC_CONCEAL
+#define calloc_conceal calloc
+#endif
+
 #ifndef HAVE_RES_HNOK
 int res_hnok(const char *);
 #endif
@@ -245,6 +258,10 @@ int res_hnok(const char *);
 #define WAIT_MYPGRP 0
 #endif
 
+#if !HAVE_DECL_IPPORT_HILASTAUTO
+#define IPPORT_HILASTAUTO 65535
+#endif
+
 #ifndef HAVE_FLOCK
 int flock(int, int);
 #endif
@@ -265,5 +282,12 @@ ssize_t getline(char **, size_t *, FILE *);
 int crypt_checkpass(const char *, const char *);
 #endif
 
+#ifndef HAVE_STRNDUP
+char * strndup(const char *, size_t);
+#endif
+
+#ifndef HAVE_STRNLEN
+char * strnlen(const char *, size_t);
+#endif
 
 #endif /* _OPENBSD_COMPAT_H */
