@@ -1,4 +1,4 @@
-/*	$OpenBSD: aliases.c,v 1.71 2016/08/31 10:18:08 gilles Exp $	*/
+/*	$OpenBSD: aliases.c,v 1.77 2018/12/28 12:47:28 eric Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -51,19 +51,19 @@ aliases_get(struct expand *expand, const char *username)
 	size_t			nbaliases;
 	int			ret;
 	union lookup		lk;
+	struct dispatcher      *dsp;
 	struct table	       *mapping = NULL;
-	struct table	       *userbase = NULL;
 	char		       *pbuf;
 
-	mapping = expand->rule->r_mapping;
-	userbase = expand->rule->r_userbase;
+	dsp = dict_xget(env->sc_dispatchers, expand->rule->dispatcher);
+	mapping = table_find(env, dsp->u.local.table_alias);
 
 	xlowercase(buf, username, sizeof(buf));
 
 	/* first, check if entry has a user-part tag */
 	pbuf = strchr(buf, *env->sc_subaddressing_delim);
 	if (pbuf) {
-		ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+		ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 		if (ret < 0)
 			return (-1);
 		if (ret)
@@ -72,7 +72,7 @@ aliases_get(struct expand *expand, const char *username)
 	}
 
 	/* no user-part tag, try looking up user */
-	ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+	ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 	if (ret <= 0)
 		return ret;
 
@@ -84,8 +84,6 @@ expand:
 			nbaliases += aliases_expand_include(expand,
 			    xn->u.buffer);
 		else {
-			xn->mapping = mapping;
-			xn->userbase = userbase;
 			expand_insert(expand, xn);
 			nbaliases++;
 		}
@@ -109,11 +107,11 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 	char		       *pbuf;
 	int			nbaliases;
 	int			ret;
+	struct dispatcher      *dsp;
 	struct table	       *mapping = NULL;
-	struct table	       *userbase = NULL;
 
-	mapping = expand->rule->r_mapping;
-	userbase = expand->rule->r_userbase;
+	dsp = dict_xget(env->sc_dispatchers, expand->rule->dispatcher);
+	mapping = table_find(env, dsp->u.local.table_virtual);
 
 	if (!bsnprintf(user, sizeof(user), "%s", maddr->user))
 		return 0;
@@ -136,7 +134,7 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 		if (!bsnprintf(buf, sizeof(buf), "%s%c%s@%s",
 			user, *env->sc_subaddressing_delim, tag, domain))
 			return 0;
-		ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+		ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 		if (ret < 0)
 			return (-1);
 		if (ret)
@@ -146,7 +144,7 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 	/* then, check if entry exists without user-part tag */
 	if (!bsnprintf(buf, sizeof(buf), "%s@%s", user, domain))
 		return 0;
-	ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+	ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 	if (ret < 0)
 		return (-1);
 	if (ret)
@@ -157,7 +155,7 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 		if (!bsnprintf(buf, sizeof(buf), "%s%c%s",
 			user, *env->sc_subaddressing_delim, tag))
 			return 0;
-		ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+		ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 		if (ret < 0)
 			return (-1);
 		if (ret)
@@ -167,7 +165,7 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 	/* Failed ? We lookup for username only */
 	if (!bsnprintf(buf, sizeof(buf), "%s", user))
 		return 0;
-	ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+	ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 	if (ret < 0)
 		return (-1);
 	if (ret)
@@ -176,14 +174,14 @@ aliases_virtual_get(struct expand *expand, const struct mailaddr *maddr)
 	if (!bsnprintf(buf, sizeof(buf), "@%s", domain))
 		return 0;
 	/* Failed ? We lookup for catch all for virtual domain */
-	ret = table_lookup(mapping, NULL, buf, K_ALIAS, &lk);
+	ret = table_lookup(mapping, K_ALIAS, buf, &lk);
 	if (ret < 0)
 		return (-1);
 	if (ret)
 		goto expand;
 
 	/* Failed ? We lookup for a *global* catch all */
-	ret = table_lookup(mapping, NULL, "@", K_ALIAS, &lk);
+	ret = table_lookup(mapping, K_ALIAS, "@", &lk);
 	if (ret <= 0)
 		return (ret);
 
@@ -195,8 +193,6 @@ expand:
 			nbaliases += aliases_expand_include(expand,
 			    xn->u.buffer);
 		else {
-			xn->mapping = mapping;
-			xn->userbase = userbase;
 			expand_insert(expand, xn);
 			nbaliases++;
 		}
