@@ -1,4 +1,4 @@
-/*	$OpenBSD: bounce.c,v 1.72 2016/02/03 05:57:09 sunil Exp $	*/
+/*	$OpenBSD: bounce.c,v 1.80 2018/12/08 08:01:15 sunil Exp $	*/
 
 /*
  * Copyright (c) 2009 Gilles Chehade <gilles@poolp.org>
@@ -82,8 +82,7 @@ struct bounce_session {
 	struct bounce_message		*msg;
 	FILE				*msgfp;
 	int				 state;
-	struct iobuf			 iobuf;
-	struct io			 io;
+	struct io			*io;
 	uint64_t			 boundary;
 };
 
@@ -99,7 +98,7 @@ static int  bounce_next_message(struct bounce_session *);
 static int  bounce_next(struct bounce_session *);
 static void bounce_delivery(struct bounce_message *, int, const char *);
 static void bounce_status(struct bounce_session *, const char *, ...);
-static void bounce_io(struct io *, int);
+static void bounce_io(struct io *, int, void *);
 static void bounce_timeout(int, short, void *);
 static void bounce_free(struct bounce_session *);
 static const char *action_str(const struct delivery_bounce *);
@@ -153,29 +152,29 @@ bounce_add(uint64_t evpid)
 
 	switch (evp.esc_class) {
 	case ESC_STATUS_OK:
-		key.bounce.type = B_DSN;
+		key.bounce.type = B_DELIVERED;
 		break;
 	case ESC_STATUS_TEMPFAIL:
-		key.bounce.type = B_WARNING;
+		key.bounce.type = B_DELAYED;
 		break;
 	default:
-		key.bounce.type = B_ERROR;
+		key.bounce.type = B_FAILED;
 	}
 
 	key.bounce.dsn_ret = evp.dsn_ret;
-	key.bounce.expire = evp.expire;
+	key.bounce.ttl = evp.ttl;
 	msg = SPLAY_FIND(bounce_message_tree, &messages, &key);
 	if (msg == NULL) {
-		msg = xcalloc(1, sizeof(*msg), "bounce_add");
+		msg = xcalloc(1, sizeof(*msg));
 		msg->msgid = key.msgid;
 		msg->bounce = key.bounce;
 
 		TAILQ_INIT(&msg->envelopes);
 
-		msg->smtpname = xstrdup(evp.smtpname, "bounce_add");
+		msg->smtpname = xstrdup(evp.smtpname);
 		(void)snprintf(buf, sizeof(buf), "%s@%s", evp.sender.user,
 		    evp.sender.domain);
-		msg->to = xstrdup(buf, "bounce_add");
+		msg->to = xstrdup(buf);
 		nmessage += 1;
 		SPLAY_INSERT(bounce_message_tree, &messages, msg);
 		log_debug("debug: bounce: new message %08" PRIx32,
@@ -190,9 +189,9 @@ bounce_add(uint64_t evpid)
 	(void)snprintf(buf, sizeof(buf), "%s@%s: %s\n", evp.dest.user,
 	    evp.dest.domain, line);
 
-	be = xmalloc(sizeof *be, "bounce_add");
+	be = xmalloc(sizeof *be);
 	be->id = evpid;
-	be->report = xstrdup(buf, "bounce_add");
+	be->report = xstrdup(buf);
 	(void)strlcpy(be->dest.user, evp.dest.user, sizeof(be->dest.user));
 	(void)strlcpy(be->dest.domain, evp.dest.domain,
 	    sizeof(be->dest.domain));
@@ -228,13 +227,14 @@ bounce_fd(int fd)
 
 	msg = TAILQ_FIRST(&pending);
 
-	s = xcalloc(1, sizeof(*s), "bounce_fd");
-	s->smtpname = xstrdup(msg->smtpname, "bounce_fd");
+	s = xcalloc(1, sizeof(*s));
+	s->smtpname = xstrdup(msg->smtpname);
 	s->state = BOUNCE_EHLO;
-	iobuf_xinit(&s->iobuf, 0, 0, "bounce_run");
-	io_init(&s->io, fd, s, bounce_io, &s->iobuf);
-	io_set_timeout(&s->io, 30000);
-	io_set_read(&s->io);
+	s->io = io_new();
+	io_set_callback(s->io, bounce_io, s);
+	io_set_fd(s->io, fd);
+	io_set_timeout(s->io, 30000);
+	io_set_read(s->io);
 	s->boundary = generate_uid();
 
 	log_debug("debug: bounce: new session %p", s);
@@ -313,7 +313,7 @@ bounce_send(struct bounce_session *s, const char *fmt, ...)
 
 	log_trace(TRACE_BOUNCE, "bounce: %p: >>> %s", s, p);
 
-	iobuf_xfqueue(&s->iobuf, "bounce_send", "%s\n", p);
+	io_xprintf(s->io, "%s\n", p);
 
 	free(p);
 }
@@ -452,7 +452,7 @@ bounce_next(struct bounce_session *s)
 	case BOUNCE_DATA_NOTICE:
 		/* Construct an appropriate notice. */
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: HEADER",
+		io_xprintf(s->io,
 		    "Subject: Delivery status notification: %s\n"
 		    "From: Mailer Daemon <MAILER-DAEMON@%s>\n"
 		    "To: %s\n"
@@ -470,7 +470,7 @@ bounce_next(struct bounce_session *s)
 		    s->boundary,
 		    s->smtpname);
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+		io_xprintf(s->io,
 		    "--%16" PRIu64 "/%s\n"
 		    "Content-Description: Notification\n"
 		    "Content-Type: text/plain; charset=us-ascii\n"
@@ -480,18 +480,15 @@ bounce_next(struct bounce_session *s)
 		    s->boundary, s->smtpname);
 
 		switch (s->msg->bounce.type) {
-		case B_ERROR:
-			iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
-			    notice_error);
+		case B_FAILED:
+			io_xprint(s->io, notice_error);
 			break;
-		case B_WARNING:
-			iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
-			    notice_warning,
+		case B_DELAYED:
+			io_xprintf(s->io, notice_warning,
 			    bounce_duration(s->msg->bounce.delay));
 			break;
-		case B_DSN:
-			iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
-			    s->msg->bounce.mta_without_dsn ?
+		case B_DELIVERED:
+			io_xprint(s->io, s->msg->bounce.mta_without_dsn ?
 			    notice_relay : notice_success);
 			break;
 		default:
@@ -499,35 +496,32 @@ bounce_next(struct bounce_session *s)
 		}
 
 		TAILQ_FOREACH(evp, &s->msg->envelopes, entry) {
-			iobuf_xfqueue(&s->iobuf,
-			    "bounce_next: DATA_NOTICE",
-			    "%s", evp->report);
+			io_xprint(s->io, evp->report);
 		}
-		iobuf_xfqueue(&s->iobuf, "bounce_next: DATA_NOTICE", "\n");
+		io_xprint(s->io, "\n");
 
-		if (s->msg->bounce.type == B_WARNING)
-			iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
-			    notice_warning2,
-			    bounce_duration(s->msg->bounce.expire));
+		if (s->msg->bounce.type == B_DELAYED)
+			io_xprintf(s->io, notice_warning2,
+			    bounce_duration(s->msg->bounce.ttl));
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: DATA_NOTICE",
+		io_xprintf(s->io,
 		    "    Below is a copy of the original message:\n"
 		    "\n");
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+		io_xprintf(s->io,
 		    "--%16" PRIu64 "/%s\n"
 		    "Content-Description: Delivery Report\n"
 		    "Content-Type: message/delivery-status\n"
 		    "\n",
 		    s->boundary, s->smtpname);
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+		io_xprintf(s->io,
 		    "Reporting-MTA: dns; %s\n"
 		    "\n",
 		    s->smtpname);
 
 		TAILQ_FOREACH(evp, &s->msg->envelopes, entry) {
-			iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+			io_xprintf(s->io,
 			    "Final-Recipient: rfc822; %s@%s\n"
 			    "Action: %s\n"
 			    "Status: %s\n"
@@ -539,30 +533,30 @@ bounce_next(struct bounce_session *s)
 		}
 
 		log_trace(TRACE_BOUNCE, "bounce: %p: >>> [... %zu bytes ...]",
-		    s, iobuf_queued(&s->iobuf));
+		    s, io_queued(s->io));
 
 		s->state = BOUNCE_DATA_MESSAGE;
 		break;
 
 	case BOUNCE_DATA_MESSAGE:
-		iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+		io_xprintf(s->io,
 		    "--%16" PRIu64 "/%s\n"
 		    "Content-Description: Message headers\n"
 		    "Content-Type: text/rfc822-headers\n"
 		    "\n",
 		    s->boundary, s->smtpname);
 
-		n = iobuf_queued(&s->iobuf);
-		while (iobuf_queued(&s->iobuf) < BOUNCE_HIWAT) {
+		n = io_queued(s->io);
+		while (io_queued(s->io) < BOUNCE_HIWAT) {
 			if ((len = getline(&line, &sz, s->msgfp)) == -1)
 				break;
 			if (len == 1 && line[0] == '\n' && /* end of headers */
-			    s->msg->bounce.type == B_DSN &&
+			    s->msg->bounce.type == B_DELIVERED &&
 			    s->msg->bounce.dsn_ret ==  DSN_RETHDRS) {
 				free(line);
 				fclose(s->msgfp);
 				s->msgfp = NULL;
-				iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+				io_xprintf(s->io,
 				    "\n--%16" PRIu64 "/%s--\n", s->boundary,
 				    s->smtpname);
 				bounce_send(s, ".");
@@ -570,8 +564,7 @@ bounce_next(struct bounce_session *s)
 				return (0);
 			}
 			line[len - 1] = '\0';
-			iobuf_xfqueue(&s->iobuf,
-			    "bounce_next: DATA_MESSAGE", "%s%s\n",
+			io_xprintf(s->io, "%s%s\n",
 			    (len == 2 && line[0] == '.') ? "." : "", line);
 		}
 		free(line);
@@ -585,11 +578,11 @@ bounce_next(struct bounce_session *s)
 			return (-1);
 		}
 
-		iobuf_xfqueue(&s->iobuf, "bounce_next: BODY",
+		io_xprintf(s->io,
 		    "\n--%16" PRIu64 "/%s--\n", s->boundary, s->smtpname);
 
 		log_trace(TRACE_BOUNCE, "bounce: %p: >>> [... %zu bytes ...]",
-		    s, iobuf_queued(&s->iobuf) - n);
+		    s, io_queued(s->io) - n);
 
 		if (feof(s->msgfp)) {
 			fclose(s->msgfp);
@@ -702,8 +695,7 @@ bounce_free(struct bounce_session *s)
 {
 	log_debug("debug: bounce: %p: deleting session", s);
 
-	iobuf_clear(&s->iobuf);
-	io_clear(&s->io);
+	io_free(s->io);
 
 	free(s->smtpname);
 	free(s);
@@ -714,9 +706,9 @@ bounce_free(struct bounce_session *s)
 }
 
 static void
-bounce_io(struct io *io, int evt)
+bounce_io(struct io *io, int evt, void *arg)
 {
-	struct bounce_session	*s = io->arg;
+	struct bounce_session	*s = arg;
 	const char		*error;
 	char			*line, *msg;
 	int			 cont;
@@ -728,17 +720,15 @@ bounce_io(struct io *io, int evt)
 	switch (evt) {
 	case IO_DATAIN:
 	    nextline:
-		line = iobuf_getline(&s->iobuf, &len);
-		if (line == NULL && iobuf_len(&s->iobuf) >= LINE_MAX) {
+		line = io_getline(s->io, &len);
+		if (line == NULL && io_datalen(s->io) >= LINE_MAX) {
 			bounce_status(s, "Input too long");
 			bounce_free(s);
 			return;
 		}
 
-		if (line == NULL) {
-			iobuf_normalize(&s->iobuf);
+		if (line == NULL)
 			break;
-		}
 
 		log_trace(TRACE_BOUNCE, "bounce: %p: <<< %s", s, line);
 
@@ -776,7 +766,7 @@ bounce_io(struct io *io, int evt)
 				bounce_free(s);
 				return;
 			}
-		if (iobuf_queued(&s->iobuf) == 0)
+		if (io_queued(s->io) == 0)
 			io_set_read(io);
 		break;
 
@@ -807,15 +797,15 @@ static const char *
 action_str(const struct delivery_bounce *b)
 {
 	switch (b->type) {
-	case B_ERROR:
-		return ("error");
-	case B_WARNING:
+	case B_FAILED:
+		return ("failed");
+	case B_DELAYED:
 		return ("delayed");
-	case B_DSN:
+	case B_DELIVERED:
 		if (b->mta_without_dsn)
 			return ("relayed");
 
-		return ("success");
+		return ("delivered");
 	default:
 		log_warn("warn: bounce: unknown bounce_type");
 		return ("");
