@@ -1,4 +1,4 @@
-/*	$OpenBSD: control.c,v 1.120 2017/11/27 08:35:59 sunil Exp $	*/
+/*	$OpenBSD: control.c,v 1.123 2018/05/31 21:06:12 gilles Exp $	*/
 
 /*
  * Copyright (c) 2012 Gilles Chehade <gilles@poolp.org>
@@ -104,7 +104,6 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 	case IMSG_CTL_LIST_ENVELOPES:
 	case IMSG_CTL_DISCOVER_EVPID:
 	case IMSG_CTL_DISCOVER_MSGID:
-	case IMSG_CTL_UNCORRUPT_MSGID:
 	case IMSG_CTL_MTA_SHOW_HOSTS:
 	case IMSG_CTL_MTA_SHOW_RELAYS:
 	case IMSG_CTL_MTA_SHOW_ROUTES:
@@ -257,8 +256,10 @@ control(void)
 
 	control_listen();
 
+#if HAVE_PLEDGE
 	if (pledge("stdio unix recvfd sendfd", NULL) == -1)
 		err(1, "pledge");
+#endif
 
 	event_dispatch();
 	fatalx("exited event loop");
@@ -316,7 +317,7 @@ control_accept(int listenfd, short event, void *arg)
 
 	count = tree_get(&ctl_count, euid);
 	if (count == NULL) {
-		count = xcalloc(1, sizeof *count, "control_accept");
+		count = xcalloc(1, sizeof *count);
 		tree_xset(&ctl_count, euid, count);
 	}
 
@@ -332,13 +333,15 @@ control_accept(int listenfd, short event, void *arg)
 		++connid;
 	} while (tree_get(&ctl_conns, connid));
 
-	c = xcalloc(1, sizeof(*c), "control_accept");
+	c = xcalloc(1, sizeof(*c));
 	c->euid = euid;
 	c->egid = egid;
 	c->id = connid;
 	c->mproc.proc = PROC_CLIENT;
 	c->mproc.handler = control_dispatch_ext;
 	c->mproc.data = c;
+	if ((c->mproc.name = strdup(proc_title(c->mproc.proc))) == NULL)
+		fatal("strdup");
 	mproc_init(&c->mproc, connfd);
 	mproc_enable(&c->mproc);
 	tree_xset(&ctl_conns, c->id, c);
@@ -752,7 +755,6 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		return;
 
 	case IMSG_CTL_DISCOVER_MSGID:
-	case IMSG_CTL_UNCORRUPT_MSGID:
 		if (c->euid)
 			goto badcred;
 
