@@ -1,4 +1,4 @@
-/*	$OpenBSD: enqueue.c,v 1.113 2016/07/03 14:30:33 gilles Exp $	*/
+/*	$OpenBSD: enqueue.c,v 1.116 2019/07/02 09:36:20 martijn Exp $	*/
 
 /*
  * Copyright (c) 2005 Henning Brauer <henning@bulabula.org>
@@ -173,8 +173,6 @@ enqueue(int argc, char *argv[], FILE *ofp)
 	FILE			*fp = NULL, *fout;
 	size_t			 sz = 0, envid_sz = 0;
 	ssize_t			 len;
-	int			 fd;
-	char			 sfn[] = "/tmp/smtpd.XXXXXXXXXX";
 	char			*line;
 	int			 dotted;
 	int			 inheaders = 1;
@@ -248,7 +246,7 @@ enqueue(int argc, char *argv[], FILE *ofp)
 		if ((pw = getpwuid(getuid())) == NULL)
 			user = "anonymous";
 		if (pw != NULL)
-			user = xstrdup(pw->pw_name, "enqueue");
+			user = xstrdup(pw->pw_name);
 	}
 	else {
 		uid_t ruid = getuid();
@@ -260,7 +258,7 @@ enqueue(int argc, char *argv[], FILE *ofp)
 		} else if ((pw = getpwuid(ruid)) == NULL) {
 			user = "anonymous";
 		}
-		user = xstrdup(pw ? pw->pw_name : user, "enqueue");
+		user = xstrdup(pw ? pw->pw_name : user);
 	}
 
 	build_from(fake_from, pw);
@@ -271,16 +269,9 @@ enqueue(int argc, char *argv[], FILE *ofp)
 		argc--;
 	}
 
-	if ((fd = mkstemp(sfn)) == -1 ||
-	    (fp = fdopen(fd, "w+")) == NULL) {
-		int saved_errno = errno;
-		if (fd != -1) {
-			unlink(sfn);
-			close(fd);
-		}
-		errc(EX_UNAVAILABLE, saved_errno, "mkstemp");
-	}
-	unlink(sfn);
+	if ((fp = tmpfile()) == NULL)
+		err(EX_UNAVAILABLE, "tmpfile");
+
 	msg.noheader = parse_message(stdin, fake_from == NULL, tflag, fp);
 
 	if (msg.rcpt_cnt == 0)
@@ -293,17 +284,20 @@ enqueue(int argc, char *argv[], FILE *ofp)
 	/* If the server is not running, enqueue the message offline */
 
 	if (!srv_connected()) {
+#if HAVE_PLEDGE
 		if (pledge("stdio", NULL) == -1)
 			err(1, "pledge");
-
+#endif
 		return (enqueue_offline(save_argc, save_argv, fp, ofp));
 	}
 
 	if ((msg.fd = open_connection()) == -1)
 		errx(EX_UNAVAILABLE, "server too busy");
 
+#if HAVE_PLEDGE
 	if (pledge("stdio wpath cpath", NULL) == -1)
 		err(1, "pledge");
+#endif
 
 	fout = fdopen(msg.fd, "a+");
 	if (fout == NULL)
@@ -545,7 +539,7 @@ build_from(char *fake_from, struct passwd *pw)
 			if (fake_from[strlen(fake_from) - 1] != '>')
 				errx(1, "leading < but no trailing >");
 			fake_from[strlen(fake_from) - 1] = 0;
-			p = xstrdup(fake_from + 1, "build_from");
+			p = xstrdup(fake_from + 1);
 
 			msg.from = qualify_addr(p);
 			free(p);
@@ -718,8 +712,7 @@ parse_addr(char *s, size_t len, int is_from)
 		if (!pstate.quote && pstate.comment && s[pos] == ')')
 			pstate.comment--;
 
-		if (!pstate.esc && !pstate.comment && !pstate.quote &&
-		    s[pos] == '\\')
+		if (!pstate.esc && !pstate.comment && s[pos] == '\\')
 			pstate.esc = 1;
 		else
 			pstate.esc = 0;
@@ -761,7 +754,7 @@ qualify_addr(char *in)
 		if (asprintf(&out, "%s@%s", in, host) == -1)
 			err(1, "qualify asprintf");
 	} else
-		out = xstrdup(in, "qualify_addr");
+		out = xstrdup(in);
 
 	return (out);
 }
