@@ -1,4 +1,4 @@
-/*	$OpenBSD: lka.c,v 1.202 2018/01/03 11:12:21 sunil Exp $	*/
+/*	$OpenBSD: lka.c,v 1.240 2019/08/28 15:50:36 martijn Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -60,21 +60,16 @@ static int lka_userinfo(const char *, const char *, struct userinfo *);
 static int lka_addrname(const char *, const struct sockaddr *,
     struct addrname *);
 static int lka_mailaddrmap(const char *, const char *, const struct mailaddr *);
-static int lka_X509_verify(struct ca_vrfy_req_msg *, const char *, const char *);
-static void lka_certificate_verify(enum imsg_type, struct ca_vrfy_req_msg *);
-static void lka_certificate_verify_resume(enum imsg_type, struct ca_vrfy_req_msg *);
+
+static void proc_timeout(int fd, short event, void *p);
+
+struct event	 ev_proc_ready;
 
 static void
 lka_imsg(struct mproc *p, struct imsg *imsg)
 {
 	struct table		*table;
 	int			 ret;
-	struct pki		*pki;
-	struct iovec		iov[2];
-	static struct ca_vrfy_req_msg	*req_ca_vrfy = NULL;
-	struct ca_vrfy_req_msg		*req_ca_vrfy_chain;
-	struct ca_cert_req_msg		*req_ca_cert;
-	struct ca_cert_resp_msg		 resp_ca_cert;
 	struct sockaddr_storage	 ss;
 	struct userinfo		 userinfo;
 	struct addrname		 addrname;
@@ -83,18 +78,49 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 	struct msg		 m;
 	union lookup		 lk;
 	char			 buf[LINE_MAX];
-	const char		*tablename, *username, *password, *label;
+	const char		*tablename, *username, *password, *label, *procname;
 	uint64_t		 reqid;
 	int			 v;
+	struct timeval		 tv;
+	const char		*direction;
+	const char		*rdns;
+	const char		*command;
+	const char		*response;
+	const char		*ciphers;
+	const char		*address;
+	const char		*domain;
+	const char		*helomethod;
+	const char		*heloname;
+	const char		*filter_name;
+	const char		*result;
+	struct sockaddr_storage	ss_src, ss_dest;
+	int                      filter_response;
+	int                      filter_phase;
+	const char              *filter_param;
+	uint32_t		 msgid;
+	uint64_t		 evpid;
+	size_t			 msgsz;
+	int			 ok;
+	int			 fcrdns;
 
 	if (imsg == NULL)
 		lka_shutdown();
 
 	switch (imsg->hdr.type) {
 
+	case IMSG_GETADDRINFO:
+	case IMSG_GETNAMEINFO:
+	case IMSG_RES_QUERY:
+		resolver_dispatch_request(p, imsg);
+		return;
+
+	case IMSG_CERT_INIT:
+	case IMSG_CERT_CERTIFICATE:
+	case IMSG_CERT_VERIFY:
+		cert_dispatch_request(p, imsg);
+		return;
+
 	case IMSG_MTA_DNS_HOST:
-	case IMSG_MTA_DNS_PTR:
-	case IMSG_SMTP_DNS_PTR:
 	case IMSG_MTA_DNS_MX:
 	case IMSG_MTA_DNS_MX_PREFERENCE:
 		dns_imsg(p, imsg);
@@ -142,63 +168,6 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 		m_close(p);
 		return;
 
-	case IMSG_SMTP_TLS_INIT:
-	case IMSG_MTA_TLS_INIT:
-		req_ca_cert = imsg->data;
-		resp_ca_cert.reqid = req_ca_cert->reqid;
-
-		xlowercase(buf, req_ca_cert->name, sizeof(buf));
-		log_debug("debug: lka: looking up pki \"%s\"", buf);
-		pki = dict_get(env->sc_pki_dict, buf);
-		if (pki == NULL)
-			if (req_ca_cert->fallback)
-				pki = dict_get(env->sc_pki_dict, "*");
-		if (pki == NULL) {
-			resp_ca_cert.status = CA_FAIL;
-			m_compose(p, imsg->hdr.type, 0, 0, -1, &resp_ca_cert,
-			    sizeof(resp_ca_cert));
-			return;
-		}
-		resp_ca_cert.status = CA_OK;
-		resp_ca_cert.cert_len = pki->pki_cert_len;
-		(void)strlcpy(resp_ca_cert.name, pki->pki_name, sizeof resp_ca_cert.name);
-		iov[0].iov_base = &resp_ca_cert;
-		iov[0].iov_len = sizeof(resp_ca_cert);
-		iov[1].iov_base = pki->pki_cert;
-		iov[1].iov_len = pki->pki_cert_len;
-		m_composev(p, imsg->hdr.type, 0, 0, -1, iov, nitems(iov));
-		return;
-
-	case IMSG_SMTP_TLS_VERIFY_CERT:
-	case IMSG_MTA_TLS_VERIFY_CERT:
-		req_ca_vrfy = xmemdup(imsg->data, sizeof *req_ca_vrfy, "lka:ca_vrfy");
-		req_ca_vrfy->cert = xmemdup((char *)imsg->data +
-		    sizeof *req_ca_vrfy, req_ca_vrfy->cert_len, "lka:ca_vrfy");
-		req_ca_vrfy->chain_cert = xcalloc(req_ca_vrfy->n_chain,
-		    sizeof (unsigned char *), "lka:ca_vrfy");
-		req_ca_vrfy->chain_cert_len = xcalloc(req_ca_vrfy->n_chain,
-		    sizeof (off_t), "lka:ca_vrfy");
-		return;
-
-	case IMSG_SMTP_TLS_VERIFY_CHAIN:
-	case IMSG_MTA_TLS_VERIFY_CHAIN:
-		if (req_ca_vrfy == NULL)
-			fatalx("lka:ca_vrfy: chain without a certificate");
-		req_ca_vrfy_chain = imsg->data;
-		req_ca_vrfy->chain_cert[req_ca_vrfy->chain_offset] = xmemdup((char *)imsg->data +
-		    sizeof *req_ca_vrfy_chain, req_ca_vrfy_chain->cert_len, "lka:ca_vrfy");
-		req_ca_vrfy->chain_cert_len[req_ca_vrfy->chain_offset] = req_ca_vrfy_chain->cert_len;
-		req_ca_vrfy->chain_offset++;
-		return;
-
-	case IMSG_SMTP_TLS_VERIFY:
-	case IMSG_MTA_TLS_VERIFY:
-		if (req_ca_vrfy == NULL)
-			fatalx("lka:ca_vrfy: verify without a certificate");
-		lka_certificate_verify(imsg->hdr.type, req_ca_vrfy);
-		req_ca_vrfy = NULL;
-		return;
-
 	case IMSG_SMTP_AUTHENTICATE:
 		m_msg(&m, imsg);
 		m_get_id(&m, &reqid);
@@ -263,7 +232,7 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 		m_get_string(&m, &tablename);
 		m_end(&m);
 
-		table = table_find(tablename, NULL);
+		table = table_find(env, tablename);
 
 		m_create(p, IMSG_MTA_LOOKUP_SOURCE, 0, 0, -1);
 		m_add_id(p, reqid);
@@ -274,7 +243,7 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 			m_add_int(p, LKA_TEMPFAIL);
 		}
 		else {
-			ret = table_fetch(table, NULL, K_SOURCE, &lk);
+			ret = table_fetch(table, K_SOURCE, &lk);
 			if (ret == -1)
 				m_add_int(p, LKA_TEMPFAIL);
 			else if (ret == 0)
@@ -306,23 +275,57 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 		m_close(p);
 		return;
 
+	case IMSG_MTA_LOOKUP_SMARTHOST:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &tablename);
+		m_end(&m);
+
+		table = table_find(env, tablename);
+
+		m_create(p, IMSG_MTA_LOOKUP_SMARTHOST, 0, 0, -1);
+		m_add_id(p, reqid);
+
+		if (table == NULL) {
+			log_warn("warn: smarthost table %s missing", tablename);
+			m_add_int(p, LKA_TEMPFAIL);
+		}
+		else {
+			ret = table_fetch(table, K_RELAYHOST, &lk);
+			if (ret == -1)
+				m_add_int(p, LKA_TEMPFAIL);
+			else if (ret == 0)
+				m_add_int(p, LKA_PERMFAIL);
+			else {
+				m_add_int(p, LKA_OK);
+				m_add_string(p, lk.relayhost);
+			}
+		}
+		m_close(p);
+		return;
+
 	case IMSG_CONF_START:
 		return;
 
 	case IMSG_CONF_END:
 		if (tracing & TRACE_TABLES)
-			table_dump_all();
+			table_dump_all(env);
 
 		/* fork & exec tables that need it */
-		table_open_all();
+		table_open_all(env);
 
+#if HAVE_PLEDGE
 		/* revoke proc & exec */
-		if (pledge("stdio rpath inet dns getpw recvfd",
+		if (pledge("stdio rpath inet dns getpw recvfd sendfd",
 			NULL) == -1)
 			err(1, "pledge");
+#endif
 
-		/* Start fulfilling requests */
-		mproc_enable(p_pony);
+		/* setup proc registering task */
+		evtimer_set(&ev_proc_ready, proc_timeout, &ev_proc_ready);
+		tv.tv_sec = 0;
+		tv.tv_usec = 10;
+		evtimer_add(&ev_proc_ready, &tv);
 		return;
 
 	case IMSG_LKA_OPEN_FORWARD:
@@ -350,7 +353,7 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 
 	case IMSG_CTL_UPDATE_TABLE:
 		ret = 0;
-		table = table_find(imsg->data, NULL);
+		table = table_find(env, imsg->data);
 		if (table == NULL) {
 			log_warnx("warn: Lookup table not found: "
 			    "\"%s\"", (char *)imsg->data);
@@ -361,6 +364,286 @@ lka_imsg(struct mproc *p, struct imsg *imsg)
 		    (ret == 1) ? IMSG_CTL_OK : IMSG_CTL_FAIL,
 		    imsg->hdr.peerid, 0, -1, NULL, 0);
 		return;
+
+	case IMSG_LKA_PROCESSOR_FORK:
+		m_msg(&m, imsg);
+		m_get_string(&m, &procname);
+		m_end(&m);
+
+		m_create(p, IMSG_LKA_PROCESSOR_ERRFD, 0, 0, -1);
+		m_add_string(p, procname);
+		m_close(p);
+
+		lka_proc_forked(procname, imsg->fd);
+		return;
+
+	case IMSG_LKA_PROCESSOR_ERRFD:
+		m_msg(&m, imsg);
+		m_get_string(&m, &procname);
+		m_end(&m);
+
+		lka_proc_errfd(procname, imsg->fd);
+		shutdown(imsg->fd, SHUT_WR);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_CONNECT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &rdns);
+		m_get_int(&m, &fcrdns);
+		m_get_sockaddr(&m, (struct sockaddr *)&ss_src);
+		m_get_sockaddr(&m, (struct sockaddr *)&ss_dest);
+		m_end(&m);
+
+		lka_report_smtp_link_connect(direction, &tv, reqid, rdns, fcrdns, &ss_src, &ss_dest);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_GREETING:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &domain);
+		m_end(&m);
+
+		lka_report_smtp_link_greeting(direction, reqid, &tv, domain);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_DISCONNECT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_end(&m);
+
+		lka_report_smtp_link_disconnect(direction, &tv, reqid);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_IDENTIFY:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &helomethod);
+		m_get_string(&m, &heloname);
+		m_end(&m);
+
+		lka_report_smtp_link_identify(direction, &tv, reqid, helomethod, heloname);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_TLS:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &ciphers);
+		m_end(&m);
+
+		lka_report_smtp_link_tls(direction, &tv, reqid, ciphers);
+		return;
+
+	case IMSG_REPORT_SMTP_LINK_AUTH:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &username);
+		m_get_string(&m, &result);
+		m_end(&m);
+
+		lka_report_smtp_link_auth(direction, &tv, reqid, username, result);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_RESET:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_end(&m);
+
+		lka_report_smtp_tx_reset(direction, &tv, reqid, msgid);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_BEGIN:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_end(&m);
+
+		lka_report_smtp_tx_begin(direction, &tv, reqid, msgid);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_MAIL:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_get_string(&m, &address);
+		m_get_int(&m, &ok);
+		m_end(&m);
+
+		lka_report_smtp_tx_mail(direction, &tv, reqid, msgid, address, ok);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_RCPT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_get_string(&m, &address);
+		m_get_int(&m, &ok);
+		m_end(&m);
+
+		lka_report_smtp_tx_rcpt(direction, &tv, reqid, msgid, address, ok);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_ENVELOPE:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_get_id(&m, &evpid);
+		m_end(&m);
+
+		lka_report_smtp_tx_envelope(direction, &tv, reqid, msgid, evpid);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_DATA:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_get_int(&m, &ok);
+		m_end(&m);
+
+		lka_report_smtp_tx_data(direction, &tv, reqid, msgid, ok);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_COMMIT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_get_size(&m, &msgsz);
+		m_end(&m);
+
+		lka_report_smtp_tx_commit(direction, &tv, reqid, msgid, msgsz);
+		return;
+
+	case IMSG_REPORT_SMTP_TX_ROLLBACK:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_u32(&m, &msgid);
+		m_end(&m);
+
+		lka_report_smtp_tx_rollback(direction, &tv, reqid, msgid);
+		return;
+
+	case IMSG_REPORT_SMTP_PROTOCOL_CLIENT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &command);
+		m_end(&m);
+
+		lka_report_smtp_protocol_client(direction, &tv, reqid, command);
+		return;
+
+	case IMSG_REPORT_SMTP_PROTOCOL_SERVER:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &response);
+		m_end(&m);
+
+		lka_report_smtp_protocol_server(direction, &tv, reqid, response);
+		return;
+
+	case IMSG_REPORT_SMTP_FILTER_RESPONSE:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_get_int(&m, &filter_phase);
+		m_get_int(&m, &filter_response);
+		m_get_string(&m, &filter_param);
+		m_end(&m);
+
+		lka_report_smtp_filter_response(direction, &tv, reqid,
+		    filter_phase, filter_response, filter_param);
+		return;
+
+	case IMSG_REPORT_SMTP_TIMEOUT:
+		m_msg(&m, imsg);
+		m_get_string(&m, &direction);
+		m_get_timeval(&m, &tv);
+		m_get_id(&m, &reqid);
+		m_end(&m);
+
+		lka_report_smtp_timeout(direction, &tv, reqid);
+		return;
+
+	case IMSG_FILTER_SMTP_PROTOCOL:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_int(&m, &filter_phase);
+		m_get_string(&m, &filter_param);
+		m_end(&m);
+
+		lka_filter_protocol(reqid, filter_phase, filter_param);
+		return;
+
+	case IMSG_FILTER_SMTP_BEGIN:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &filter_name);
+		m_get_sockaddr(&m, (struct sockaddr *)&ss_src);
+		m_get_sockaddr(&m, (struct sockaddr *)&ss_dest);
+		m_get_string(&m, &rdns);
+		m_get_int(&m, &fcrdns);
+		m_end(&m);
+
+		lka_filter_begin(reqid, filter_name, &ss_src, &ss_dest, rdns, fcrdns);
+		return;
+
+	case IMSG_FILTER_SMTP_END:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_end(&m);
+
+		lka_filter_end(reqid);
+		return;
+
+	case IMSG_FILTER_SMTP_DATA_BEGIN:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_end(&m);
+
+		lka_filter_data_begin(reqid);
+		return;
+
+	case IMSG_FILTER_SMTP_DATA_END:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_end(&m);
+
+		lka_filter_data_end(reqid);
+		return;
+
 	}
 
 	errx(1, "lka_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
@@ -426,9 +709,14 @@ lka(void)
 	/* Ignore them until we get our config */
 	mproc_disable(p_pony);
 
+	lka_report_init();
+	lka_filter_init();
+
+#if HAVE_PLEDGE
 	/* proc & exec will be revoked before serving requests */
-	if (pledge("stdio rpath inet dns getpw recvfd proc exec", NULL) == -1)
+	if (pledge("stdio rpath inet dns getpw recvfd sendfd proc exec", NULL) == -1)
 		err(1, "pledge");
+#endif
 
 	event_dispatch();
 	fatalx("exited event loop");
@@ -436,6 +724,26 @@ lka(void)
 	return (0);
 }
 
+static void
+proc_timeout(int fd, short event, void *p)
+{
+	struct event	*ev = p;
+	struct timeval	 tv;
+
+	if (!lka_proc_ready())
+		goto reset;
+
+	lka_filter_ready();
+	mproc_enable(p_pony);
+	return;
+
+reset:
+	tv.tv_sec = 0;
+	tv.tv_usec = 10;
+	evtimer_add(ev, &tv);
+}
+
+
 static int
 lka_authenticate(const char *tablename, const char *user, const char *password)
 {
@@ -443,14 +751,14 @@ lka_authenticate(const char *tablename, const char *user, const char *password)
 	union lookup		 lk;
 
 	log_debug("debug: lka: authenticating for %s:%s", tablename, user);
-	table = table_find(tablename, NULL);
+	table = table_find(env, tablename);
 	if (table == NULL) {
 		log_warnx("warn: could not find table %s needed for authentication",
 		    tablename);
 		return (LKA_TEMPFAIL);
 	}
 
-	switch (table_lookup(table, NULL, user, K_CREDENTIALS, &lk)) {
+	switch (table_lookup(table, K_CREDENTIALS, user, &lk)) {
 	case -1:
 		log_warnx("warn: user credentials lookup fail for %s:%s",
 		    tablename, user);
@@ -472,7 +780,7 @@ lka_credentials(const char *tablename, const char *label, char *dst, size_t sz)
 	char			*buf;
 	int			 buflen, r;
 
-	table = table_find(tablename, NULL);
+	table = table_find(env, tablename);
 	if (table == NULL) {
 		log_warnx("warn: credentials table %s missing", tablename);
 		return (LKA_TEMPFAIL);
@@ -480,7 +788,7 @@ lka_credentials(const char *tablename, const char *label, char *dst, size_t sz)
 
 	dst[0] = '\0';
 
-	switch(table_lookup(table, NULL, label, K_CREDENTIALS, &lk)) {
+	switch (table_lookup(table, K_CREDENTIALS, label, &lk)) {
 	case -1:
 		log_warnx("warn: credentials lookup fail for %s:%s",
 		    tablename, label);
@@ -515,13 +823,13 @@ lka_userinfo(const char *tablename, const char *username, struct userinfo *res)
 	union lookup	 lk;
 
 	log_debug("debug: lka: userinfo %s:%s", tablename, username);
-	table = table_find(tablename, NULL);
+	table = table_find(env, tablename);
 	if (table == NULL) {
 		log_warnx("warn: cannot find user table %s", tablename);
 		return (LKA_TEMPFAIL);
 	}
 
-	switch (table_lookup(table, NULL, username, K_USERINFO, &lk)) {
+	switch (table_lookup(table, K_USERINFO, username, &lk)) {
 	case -1:
 		log_warnx("warn: failure during userinfo lookup %s:%s",
 		    tablename, username);
@@ -545,13 +853,13 @@ lka_addrname(const char *tablename, const struct sockaddr *sa,
 	source = sa_to_text(sa);
 
 	log_debug("debug: lka: helo %s:%s", tablename, source);
-	table = table_find(tablename, NULL);
+	table = table_find(env, tablename);
 	if (table == NULL) {
 		log_warnx("warn: cannot find helo table %s", tablename);
 		return (LKA_TEMPFAIL);
 	}
 
-	switch (table_lookup(table, NULL, source, K_ADDRNAME, &lk)) {
+	switch (table_lookup(table, K_ADDRNAME, source, &lk)) {
 	case -1:
 		log_warnx("warn: failure during helo lookup %s:%s",
 		    tablename, source);
@@ -573,13 +881,13 @@ lka_mailaddrmap(const char *tablename, const char *username, const struct mailad
 	int			found;
 
 	log_debug("debug: lka: mailaddrmap %s:%s", tablename, username);
-	table = table_find(tablename, NULL);
+	table = table_find(env, tablename);
 	if (table == NULL) {
 		log_warnx("warn: cannot find mailaddrmap table %s", tablename);
 		return (LKA_TEMPFAIL);
 	}
 
-	switch (table_lookup(table, NULL, username, K_MAILADDRMAP, &lk)) {
+	switch (table_lookup(table, K_MAILADDRMAP, username, &lk)) {
 	case -1:
 		log_warnx("warn: failure during mailaddrmap lookup %s:%s",
 		    tablename, username);
@@ -601,88 +909,3 @@ lka_mailaddrmap(const char *tablename, const char *username, const struct mailad
 	}
 	return (LKA_OK);
 }
-
-static int
-lka_X509_verify(struct ca_vrfy_req_msg *vrfy,
-    const char *CAfile, const char *CRLfile)
-{
-	X509			*x509;
-	X509			*x509_tmp;
-	STACK_OF(X509)		*x509_chain;
-	const unsigned char    	*d2i;
-	size_t			i;
-	int			ret = 0;
-	const char		*errstr;
-
-	x509 = NULL;
-	x509_tmp = NULL;
-	x509_chain = NULL;
-
-	d2i = vrfy->cert;
-	if (d2i_X509(&x509, &d2i, vrfy->cert_len) == NULL) {
-		x509 = NULL;
-		goto end;
-	}
-
-	if (vrfy->n_chain) {
-		x509_chain = sk_X509_new_null();
-		for (i = 0; i < vrfy->n_chain; ++i) {
-			d2i = vrfy->chain_cert[i];
-			if (d2i_X509(&x509_tmp, &d2i, vrfy->chain_cert_len[i]) == NULL)
-				goto end;
-			sk_X509_insert(x509_chain, x509_tmp, i);
-			x509_tmp = NULL;
-		}
-	}
-	if (!ca_X509_verify(x509, x509_chain, CAfile, NULL, &errstr))
-		log_debug("debug: lka: X509 verify: %s", errstr);
-	else
-		ret = 1;
-
-end:
-	X509_free(x509);
-	X509_free(x509_tmp);
-	if (x509_chain)
-		sk_X509_pop_free(x509_chain, X509_free);
-
-	return ret;
-}
-
-static void
-lka_certificate_verify(enum imsg_type type, struct ca_vrfy_req_msg *req)
-{
-	lka_certificate_verify_resume(type, req);
-}
-
-static void
-lka_certificate_verify_resume(enum imsg_type type, struct ca_vrfy_req_msg *req)
-{
-	struct ca_vrfy_resp_msg		resp;
-	struct ca		       *sca;
-	const char		       *cafile;
-	size_t				i;
-
-	resp.reqid = req->reqid;
-	sca = dict_get(env->sc_ca_dict, req->name);
-	if (sca == NULL)
-		if (req->fallback)
-			sca = dict_get(env->sc_ca_dict, "*");
-	cafile = sca ? sca->ca_cert_file : CA_FILE;
-
-	if (sca == NULL && !req->fallback)
-		resp.status = CA_FAIL;
-	else if (!lka_X509_verify(req, cafile, NULL))
-		resp.status = CA_FAIL;
-	else
-		resp.status = CA_OK;
-
-	m_compose(p_pony, type, 0, 0, -1, &resp,
-	    sizeof resp);
-
-	for (i = 0; i < req->n_chain; ++i)
-		free(req->chain_cert[i]);
-	free(req->chain_cert);
-	free(req->chain_cert_len);
-	free(req->cert);
-	free(req);
-}
