@@ -1,4 +1,4 @@
-/*	$OpenBSD: mproc.c,v 1.29 2017/03/17 20:57:57 eric Exp $	*/
+/*	$OpenBSD: mproc.c,v 1.35 2019/10/03 05:50:28 gilles Exp $	*/
 
 /*
  * Copyright (c) 2012 Eric Faurot <eric@faurot.net>
@@ -50,7 +50,7 @@ mproc_fork(struct mproc *p, const char *path, char *argv[])
 {
 	int sp[2];
 
-	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, sp) < 0)
+	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, sp) == -1)
 		return (-1);
 
 	io_set_nonblocking(sp[0]);
@@ -63,7 +63,6 @@ mproc_fork(struct mproc *p, const char *path, char *argv[])
 		/* child process */
 		dup2(sp[0], STDIN_FILENO);
 		closefrom(STDERR_FILENO + 1);
-
 		execv(path, argv);
 		err(1, "execv: %s", path);
 	}
@@ -310,7 +309,7 @@ m_add(struct mproc *p, const void *data, size_t len)
 	void	*tmp;
 
 	if (p->m_pos + len + IMSG_HEADER_SIZE > MAX_IMSGSIZE) {
-		log_warnx("warn: message to large");
+		log_warnx("warn: message too large");
 		fatal(NULL);
 	}
 
@@ -366,7 +365,8 @@ m_flush(struct mproc *p)
 
 	p->m_pos = 0;
 
-	imsg_flush(&p->imsgbuf);
+	if (imsg_flush(&p->imsgbuf) == -1)
+		fatal("imsg_flush");
 }
 
 static struct imsg * current;
@@ -439,10 +439,22 @@ m_add_time(struct mproc *m, time_t v)
 	m_add(m, &v, sizeof(v));
 };
 
+void
+m_add_timeval(struct mproc *m, struct timeval *tv)
+{
+	m_add(m, tv, sizeof(*tv));
+}
+
+
 void
 m_add_string(struct mproc *m, const char *v)
 {
-	m_add(m, v, strlen(v) + 1);
+	if (v) {
+		m_add(m, "s", 1);
+		m_add(m, v, strlen(v) + 1);
+	}
+	else
+		m_add(m, "\0", 1);
 };
 
 void
@@ -536,14 +548,29 @@ m_get_time(struct msg *m, time_t *t)
 	m_get(m, t, sizeof(*t));
 }
 
+void
+m_get_timeval(struct msg *m, struct timeval *tv)
+{
+	m_get(m, tv, sizeof(*tv));
+}
+
 void
 m_get_string(struct msg *m, const char **s)
 {
 	uint8_t	*end;
+	char c;
 
 	if (m->pos >= m->end)
 		m_error("msg too short");
 
+	c = *m->pos++;
+	if (c == '\0') {
+		*s = NULL;
+		return;
+	}
+
+	if (m->pos >= m->end)
+		m_error("msg too short");
 	end = memchr(m->pos, 0, m->end - m->pos);
 	if (end == NULL)
 		m_error("unterminated string");
@@ -557,6 +584,11 @@ m_get_data(struct msg *m, const void **data, size_t *sz)
 {
 	m_get_size(m, sz);
 
+	if (*sz == 0) {
+		*data = NULL;
+		return;
+	}
+
 	if (m->pos + *sz > m->end)
 		m_error("msg too short");
 
