@@ -1,4 +1,4 @@
-/*	$OpenBSD: pony.c,v 1.18 2017/08/13 11:10:30 eric Exp $	*/
+/*	$OpenBSD: pony.c,v 1.27 2019/06/13 11:45:35 eric Exp $	*/
 
 /*
  * Copyright (c) 2014 Gilles Chehade <gilles@poolp.org>
@@ -38,7 +38,6 @@
 #include <unistd.h>
 #include <limits.h>
 #include <grp.h>
-#include <vis.h>
 
 #include "smtpd.h"
 #include "log.h"
@@ -59,6 +58,19 @@ pony_imsg(struct mproc *p, struct imsg *imsg)
 		pony_shutdown();
 
 	switch (imsg->hdr.type) {
+
+	case IMSG_GETADDRINFO:
+	case IMSG_GETADDRINFO_END:
+	case IMSG_GETNAMEINFO:
+	case IMSG_RES_QUERY:
+		resolver_dispatch_result(p, imsg);
+		return;
+
+	case IMSG_CERT_INIT:
+	case IMSG_CERT_VERIFY:
+		cert_dispatch_result(p, imsg);
+		return;
+
 	case IMSG_CONF_START:
 		return;
 	case IMSG_CONF_END:
@@ -78,16 +90,15 @@ pony_imsg(struct mproc *p, struct imsg *imsg)
 		return;
 
 	/* smtp imsg */
-	case IMSG_SMTP_DNS_PTR:
 	case IMSG_SMTP_CHECK_SENDER:
 	case IMSG_SMTP_EXPAND_RCPT:
 	case IMSG_SMTP_LOOKUP_HELO:
 	case IMSG_SMTP_AUTHENTICATE:
-	case IMSG_SMTP_TLS_INIT:
-	case IMSG_SMTP_TLS_VERIFY:
 	case IMSG_SMTP_MESSAGE_COMMIT:
 	case IMSG_SMTP_MESSAGE_CREATE:
 	case IMSG_SMTP_MESSAGE_OPEN:
+	case IMSG_FILTER_SMTP_PROTOCOL:
+	case IMSG_FILTER_SMTP_DATA_BEGIN:
 	case IMSG_QUEUE_ENVELOPE_SUBMIT:
 	case IMSG_QUEUE_ENVELOPE_COMMIT:
 	case IMSG_QUEUE_SMTP_SESSION:
@@ -101,14 +112,12 @@ pony_imsg(struct mproc *p, struct imsg *imsg)
 	case IMSG_QUEUE_TRANSFER:
 	case IMSG_MTA_OPEN_MESSAGE:
 	case IMSG_MTA_LOOKUP_CREDENTIALS:
+	case IMSG_MTA_LOOKUP_SMARTHOST:
 	case IMSG_MTA_LOOKUP_SOURCE:
 	case IMSG_MTA_LOOKUP_HELO:
 	case IMSG_MTA_DNS_HOST:
 	case IMSG_MTA_DNS_HOST_END:
 	case IMSG_MTA_DNS_MX_PREFERENCE:
-	case IMSG_MTA_DNS_PTR:
-	case IMSG_MTA_TLS_INIT:
-	case IMSG_MTA_TLS_VERIFY:
 	case IMSG_CTL_RESUME_ROUTE:
 	case IMSG_CTL_MTA_SHOW_HOSTS:
 	case IMSG_CTL_MTA_SHOW_RELAYS:
@@ -191,8 +200,10 @@ pony(void)
 
 	ca_engine_init();
 
+#if HAVE_PLEDGE
 	if (pledge("stdio inet unix recvfd sendfd", NULL) == -1)
 		err(1, "pledge");
+#endif
 
 	event_dispatch();
 	fatalx("exited event loop");
