@@ -1,4 +1,4 @@
-/*	$OpenBSD: queue_backend.c,v 1.62 2016/02/04 12:46:28 eric Exp $	*/
+/*	$OpenBSD: queue_backend.c,v 1.65 2018/12/30 23:09:58 guenther Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -33,7 +33,6 @@
 #include <imsg.h>
 #include <limits.h>
 #include <inttypes.h>
-#include <libgen.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -66,8 +65,6 @@ static int (*handler_message_create)(uint32_t *);
 static int (*handler_message_commit)(uint32_t, const char*);
 static int (*handler_message_delete)(uint32_t);
 static int (*handler_message_fd_r)(uint32_t);
-static int (*handler_message_corrupt)(uint32_t);
-static int (*handler_message_uncorrupt)(uint32_t);
 static int (*handler_envelope_create)(uint32_t, const char *, size_t, uint64_t *);
 static int (*handler_envelope_delete)(uint64_t);
 static int (*handler_envelope_update)(uint64_t, const char *, size_t);
@@ -256,7 +253,6 @@ queue_message_commit(uint32_t msgid)
 		}
 	}
 
-#ifdef HAVE_GCM_CRYPTO
 	if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
 		bsnprintf(tmppath, sizeof tmppath, "%s.enc", msgpath);
 		ifp = fopen(msgpath, "r");
@@ -278,7 +274,6 @@ queue_message_commit(uint32_t msgid)
 			return (0);
 		}
 	}
-#endif
 
 	r = handler_message_commit(msgid, msgpath);
 	profile_leave();
@@ -300,27 +295,6 @@ err:
 	return 0;
 }
 
-int
-queue_message_corrupt(uint32_t msgid)
-{
-	int	r;
-
-	profile_enter("queue_message_corrupt");
-	r = handler_message_corrupt(msgid);
-	profile_leave();
-
-	log_trace(TRACE_QUEUE,
-	    "queue-backend: queue_message_corrupt(%08"PRIx32") -> %d", msgid, r);
-
-	return (r);
-}
-
-int
-queue_message_uncorrupt(uint32_t msgid)
-{
-	return handler_message_uncorrupt(msgid);
-}
-
 int
 queue_message_fd_r(uint32_t msgid)
 {
@@ -338,7 +312,6 @@ queue_message_fd_r(uint32_t msgid)
 	if (fdin == -1)
 		return (-1);
 
-#ifdef HAVE_GCM_CRYPTO
 	if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
 		if ((fdout = mktmpfile()) == -1)
 			goto err;
@@ -360,7 +333,6 @@ queue_message_fd_r(uint32_t msgid)
 		ofp = NULL;
 		lseek(fdin, SEEK_SET, 0);
 	}
-#endif
 
 	if (env->sc_queue_flags & QUEUE_COMPRESSION) {
 		if ((fdout = mktmpfile()) == -1)
@@ -417,10 +389,8 @@ queue_envelope_dump_buffer(struct envelope *ep, char *evpbuf, size_t evpbufsize)
 	size_t	evplen;
 	size_t	complen;
 	char	compbuf[sizeof(struct envelope)];
-#ifdef HAVE_GCM_CRYPTO
 	size_t	enclen;
 	char	encbuf[sizeof(struct envelope)];
-#endif
 
 	evp = evpbuf;
 	evplen = envelope_dump_buffer(ep, evpbuf, evpbufsize);
@@ -435,7 +405,6 @@ queue_envelope_dump_buffer(struct envelope *ep, char *evpbuf, size_t evpbufsize)
 		evplen = complen;
 	}
 
-#ifdef HAVE_GCM_CRYPTO
 	if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
 		enclen = crypto_encrypt_buffer(evp, evplen, encbuf, sizeof encbuf);
 		if (enclen == 0)
@@ -443,7 +412,6 @@ queue_envelope_dump_buffer(struct envelope *ep, char *evpbuf, size_t evpbufsize)
 		evp = encbuf;
 		evplen = enclen;
 	}
-#endif
 
 	memmove(evpbuf, evp, evplen);
 
@@ -457,15 +425,12 @@ queue_envelope_load_buffer(struct envelope *ep, char *evpbuf, size_t evpbufsize)
 	size_t		 evplen;
 	char		 compbuf[sizeof(struct envelope)];
 	size_t		 complen;
-#ifdef HAVE_GCM_CRYPTO
 	char		 encbuf[sizeof(struct envelope)];
 	size_t		 enclen;
-#endif
 
 	evp = evpbuf;
 	evplen = evpbufsize;
 
-#ifdef HAVE_GCM_CRYPTO
 	if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
 		enclen = crypto_decrypt_buffer(evp, evplen, encbuf, sizeof encbuf);
 		if (enclen == 0)
@@ -473,7 +438,6 @@ queue_envelope_load_buffer(struct envelope *ep, char *evpbuf, size_t evpbufsize)
 		evp = encbuf;
 		evplen = enclen;
 	}
-#endif
 
 	if (env->sc_queue_flags & QUEUE_COMPRESSION) {
 		complen = uncompress_chunk(evp, evplen, compbuf, sizeof compbuf);
@@ -494,7 +458,7 @@ queue_envelope_cache_add(struct envelope *e)
 	while (tree_count(&evpcache_tree) >= env->sc_queue_evpcache_size)
 		queue_envelope_cache_del(TAILQ_LAST(&evpcache_list, evplst)->id);
 
-	cached = xcalloc(1, sizeof *cached, "queue_envelope_cache_add");
+	cached = xcalloc(1, sizeof *cached);
 	*cached = *e;
 	TAILQ_INSERT_HEAD(&evpcache_list, cached, entry);
 	tree_xset(&evpcache_tree, e->id, cached);
@@ -621,11 +585,9 @@ queue_envelope_load(uint64_t evpid, struct envelope *ep)
 			}
 			return (1);
 		}
-		log_debug("debug: invalid envelope %016" PRIx64 ": %s",
-		    ep->id, e);
+		log_warnx("warn: invalid envelope %016" PRIx64 ": %s",
+		    evpid, e);
 	}
-
-	(void)queue_message_corrupt(evpid_to_msgid(evpid));
 	return (0);
 }
 
@@ -686,11 +648,9 @@ queue_message_walk(struct envelope *ep, uint32_t msgid, int *done, void **data)
 			 */
 			return (1);
 		}
-		log_debug("debug: invalid envelope %016" PRIx64 ": %s",
-		    ep->id, e);
-		(void)queue_message_corrupt(evpid_to_msgid(evpid));
+		log_warnx("warn: invalid envelope %016" PRIx64 ": %s",
+		    evpid, e);
 	}
-
 	return (0);
 }
 
@@ -720,11 +680,9 @@ queue_envelope_walk(struct envelope *ep)
 				queue_envelope_cache_add(ep);
 			return (1);
 		}
-		log_debug("debug: invalid envelope %016" PRIx64 ": %s",
-		    ep->id, e);
-		(void)queue_message_corrupt(evpid_to_msgid(evpid));
+		log_warnx("warn: invalid envelope %016" PRIx64 ": %s",
+		    evpid, e);
 	}
-
 	return (0);
 }
 
@@ -807,18 +765,6 @@ queue_api_on_message_fd_r(int(*cb)(uint32_t))
 	handler_message_fd_r = cb;
 }
 
-void
-queue_api_on_message_corrupt(int(*cb)(uint32_t))
-{
-	handler_message_corrupt = cb;
-}
-
-void
-queue_api_on_message_uncorrupt(int(*cb)(uint32_t))
-{
-	handler_message_uncorrupt = cb;
-}
-
 void
 queue_api_on_envelope_create(int(*cb)(uint32_t, const char *, size_t, uint64_t *))
 {
