@@ -1,4 +1,4 @@
-/*	$OpenBSD: scheduler.c,v 1.56 2017/01/09 14:49:22 reyk Exp $	*/
+/*	$OpenBSD: scheduler.c,v 1.60 2018/12/30 23:09:58 guenther Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -35,7 +35,6 @@
 #include <grp.h> /* needed for setgroups */
 #include <imsg.h>
 #include <inttypes.h>
-#include <libgen.h>
 #include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
@@ -217,9 +216,9 @@ scheduler_imsg(struct mproc *p, struct imsg *imsg)
 			    si.lastbounce < timestamp) {
 	    			req.evpid = evp.id;
 				req.timestamp = timestamp;
-				req.bounce.type = B_WARNING;
+				req.bounce.type = B_DELAYED;
 				req.bounce.delay = env->sc_bounce_warn[i];
-				req.bounce.expire = si.expire;
+				req.bounce.ttl = si.ttl;
 				m_compose(p, IMSG_SCHED_ENVELOPE_BOUNCE, 0, 0, -1,
 				    &req, sizeof req);
 				break;
@@ -436,7 +435,7 @@ scheduler(void)
 		errx(1, "cannot find scheduler backend \"%s\"",
 		    backend_scheduler);
 
-	purge_config(PURGE_EVERYTHING);
+	purge_config(PURGE_EVERYTHING & ~PURGE_DISPATCHERS);
 
 	if ((pw = getpwnam(SMTPD_USER)) == NULL)
 		fatalx("unknown user " SMTPD_USER);
@@ -455,10 +454,10 @@ scheduler(void)
 	    setresuid(pw->pw_uid, pw->pw_uid, pw->pw_uid))
 		fatal("scheduler: cannot drop privileges");
 
-	evpids = xcalloc(env->sc_scheduler_max_schedule, sizeof *evpids, "scheduler: init evpids");
-	types = xcalloc(env->sc_scheduler_max_schedule, sizeof *types, "scheduler: init types");
-	msgids = xcalloc(env->sc_scheduler_max_msg_batch_size, sizeof *msgids, "scheduler: list msg");
-	state = xcalloc(env->sc_scheduler_max_evp_batch_size, sizeof *state, "scheduler: list evp");
+	evpids = xcalloc(env->sc_scheduler_max_schedule, sizeof *evpids);
+	types = xcalloc(env->sc_scheduler_max_schedule, sizeof *types);
+	msgids = xcalloc(env->sc_scheduler_max_msg_batch_size, sizeof *msgids);
+	state = xcalloc(env->sc_scheduler_max_evp_batch_size, sizeof *state);
 
 	imsg_callback = scheduler_imsg;
 	event_init();
@@ -474,8 +473,10 @@ scheduler(void)
 	evtimer_set(&ev, scheduler_timeout, NULL);
 	scheduler_reset_events();
 
+#if HAVE_PLEDGE
 	if (pledge("stdio", NULL) == -1)
 		err(1, "pledge");
+#endif
 
 	event_dispatch();
 	fatalx("exited event loop");
