@@ -47,6 +47,12 @@
 #ifndef PATH_SPOOL
 #define PATH_SPOOL		"/var/spool/smtpd"
 #endif
+#ifndef PATH_MAILLOCAL
+#define	PATH_MAILLOCAL		"/usr/libexec/mail.local"
+#endif
+#ifndef PATH_MAKEMAP
+#define	PATH_MAKEMAP		"/usr/sbin/makemap"
+#endif
 
 #define SUBADDRESSING_DELIMITER	"+"
 
