@@ -1,4 +1,4 @@
-/*	$OpenBSD: to.c,v 1.28 2016/05/30 12:33:44 mpi Exp $	*/
+/*	$OpenBSD: to.c,v 1.43 2019/09/19 16:00:59 gilles Exp $	*/
 
 /*
  * Copyright (c) 2009 Jacek Masiulaniec <jacekm@dobremiasto.net>
@@ -38,7 +38,6 @@
 #include <imsg.h>
 #include <limits.h>
 #include <inttypes.h>
-#include <libgen.h>
 #include <netdb.h>
 #include <pwd.h>
 #include <stdarg.h>
@@ -52,7 +51,6 @@
 #include "log.h"
 
 static const char *in6addr_to_text(const struct in6_addr *);
-static int alias_is_maildir(struct expandnode *, const char *, size_t);
 static int alias_is_filter(struct expandnode *, const char *, size_t);
 static int alias_is_username(struct expandnode *, const char *, size_t);
 static int alias_is_address(struct expandnode *, const char *, size_t);
@@ -177,10 +175,9 @@ sa_to_text(const struct sockaddr *sa)
 		const struct in6_addr	*in6_addr;
 
 		in6 = (const struct sockaddr_in6 *)sa;
-		(void)strlcpy(buf, "IPv6:", sizeof(buf));
-		p = buf + 5;
+		p = buf;
 		in6_addr = &in6->sin6_addr;
-		(void)bsnprintf(p, NI_MAXHOST, "%s", in6addr_to_text(in6_addr));
+		(void)bsnprintf(p, NI_MAXHOST, "[%s]", in6addr_to_text(in6_addr));
 	}
 
 	return (buf);
@@ -283,6 +280,8 @@ text_to_netaddr(struct netaddr *netaddr, const char *s)
 	struct sockaddr_in	ssin;
 	struct sockaddr_in6	ssin6;
 	int			bits;
+	char			buf[NI_MAXHOST];
+	size_t			len;
 
 	memset(&ssin, 0, sizeof(struct sockaddr_in));
 	memset(&ssin6, 0, sizeof(struct sockaddr_in6));
@@ -299,12 +298,26 @@ text_to_netaddr(struct netaddr *netaddr, const char *s)
 		ss.ss_len = sizeof(struct sockaddr_in);
 #endif
 	} else {
-		bits = inet_net_pton(AF_INET6, s, &ssin6.sin6_addr,
+		if (s[0] != '[') {
+			if ((len = strlcpy(buf, s, sizeof buf)) >= sizeof buf)
+				return 0;
+		}
+		else {
+			s++;
+			if (strncasecmp("IPv6:", s, 5) == 0)
+				s += 5;
+			if ((len = strlcpy(buf, s, sizeof buf)) >= sizeof buf)
+				return 0;
+			if (buf[len-1] != ']')
+				return 0;
+			buf[len-1] = 0;
+		}
+		bits = inet_net_pton(AF_INET6, buf, &ssin6.sin6_addr,
 		    sizeof(struct in6_addr));
 		if (bits == -1) {
 			if (errno != EAFNOSUPPORT)
 				return 0;
-			bits = broken_inet_net_pton_ipv6(s, &ssin6.sin6_addr,
+			bits = broken_inet_net_pton_ipv6(buf, &ssin6.sin6_addr,
 			    sizeof(struct in6_addr));
 			if (bits == -1)
 				return 0;
@@ -326,23 +339,20 @@ text_to_relayhost(struct relayhost *relay, const char *s)
 {
 	static const struct schema {
 		const char	*name;
-		uint16_t       	 flags;
+		int		 tls;
+		uint16_t	 flags;
+		uint16_t	 port;
 	} schemas [] = {
 		/*
 		 * new schemas should be *appended* otherwise the default
 		 * schema index needs to be updated later in this function.
 		 */
-		{ "smtp://",		0				},
-		{ "lmtp://",		F_LMTP				},
-		{ "smtp+tls://",       	F_TLS_OPTIONAL 			},
-		{ "smtps://",		F_SMTPS				},
-		{ "tls://",		F_STARTTLS			},
-		{ "smtps+auth://",	F_SMTPS|F_AUTH			},
-		{ "tls+auth://",	F_STARTTLS|F_AUTH		},
-		{ "secure://",		F_SMTPS|F_STARTTLS		},
-		{ "secure+auth://",	F_SMTPS|F_STARTTLS|F_AUTH	},
-		{ "backup://",		F_BACKUP       			},
-		{ "tls+backup://",	F_BACKUP|F_STARTTLS    		}
+		{ "smtp://",		RELAY_TLS_OPPORTUNISTIC, 0,		25 },
+		{ "smtp+tls://",	RELAY_TLS_STARTTLS,	 0,		25 },
+		{ "smtp+notls://",	RELAY_TLS_NO,		 0,		25 },
+		/* need to specify an explicit port for LMTP */
+		{ "lmtp://",		RELAY_TLS_NO,		 RELAY_LMTP,	0 },
+		{ "smtps://",		RELAY_TLS_SMTPS,	 0,		465 }
 	};
 	const char     *errstr = NULL;
 	char	       *p, *q;
@@ -365,18 +375,16 @@ text_to_relayhost(struct relayhost *relay, const char *s)
 		if (strstr(buffer, "://"))
 			return 0;
 
-		/* no schema, default to smtp+tls:// */
-		i = 2;
+		/* no schema, default to smtp:// */
+		i = 0;
 		p = buffer;
 	}
 	else
 		p = buffer + strlen(schemas[i].name);
 
+	relay->tls = schemas[i].tls;
 	relay->flags = schemas[i].flags;
-
-	/* need to specify an explicit port for LMTP */
-	if (relay->flags & F_LMTP)
-		relay->port = 0;
+	relay->port = schemas[i].port;
 
 	/* first, we extract the label if any */
 	if ((q = strchr(p, '@')) != NULL) {
@@ -412,76 +420,24 @@ text_to_relayhost(struct relayhost *relay, const char *s)
 	/* finally, we extract the port */
 	p = beg + len;
 	if (*p == ':') {
-		relay->port = strtonum(p+1, 1, 0xffff, &errstr);
+		relay->port = strtonum(p+1, 1, IPPORT_HILASTAUTO, &errstr);
 		if (errstr)
 			return 0;
 	}
 
 	if (!valid_domainpart(relay->hostname))
 		return 0;
-	if ((relay->flags & F_LMTP) && (relay->port == 0))
+	if ((relay->flags & RELAY_LMTP) && (relay->port == 0))
 		return 0;
-	if (relay->authlabel[0] == '\0' && relay->flags & F_AUTH)
-		return 0;
-	if (relay->authlabel[0] != '\0' && !(relay->flags & F_AUTH))
-		return 0;
-	return 1;
-}
-
-const char *
-relayhost_to_text(const struct relayhost *relay)
-{
-	static char	buf[4096];
-	char		port[4096];
-	uint16_t	mask = F_SMTPS|F_STARTTLS|F_AUTH|F_TLS_OPTIONAL|F_LMTP|F_BACKUP;
-
-	memset(buf, 0, sizeof buf);
-	switch (relay->flags & mask) {
-	case F_SMTPS|F_STARTTLS|F_AUTH:
-		(void)strlcat(buf, "secure+auth://", sizeof buf);
-		break;
-	case F_SMTPS|F_STARTTLS:
-		(void)strlcat(buf, "secure://", sizeof buf);
-		break;
-	case F_STARTTLS|F_AUTH:
-		(void)strlcat(buf, "tls+auth://", sizeof buf);
-		break;
-	case F_SMTPS|F_AUTH:
-		(void)strlcat(buf, "smtps+auth://", sizeof buf);
-		break;
-	case F_STARTTLS:
-		(void)strlcat(buf, "tls://", sizeof buf);
-		break;
-	case F_SMTPS:
-		(void)strlcat(buf, "smtps://", sizeof buf);
-		break;
-	case F_BACKUP|F_STARTTLS:
-		(void)strlcat(buf, "tls+backup://", sizeof buf);
-		break;
-	case F_BACKUP:
-		(void)strlcat(buf, "backup://", sizeof buf);
-		break;
-	case F_TLS_OPTIONAL:
-		(void)strlcat(buf, "smtp+tls://", sizeof buf);
-		break;
-	case F_LMTP:
-		(void)strlcat(buf, "lmtp://", sizeof buf);
-		break;
-	default:
-		(void)strlcat(buf, "smtp://", sizeof buf);
-		break;
-	}
 	if (relay->authlabel[0]) {
-		(void)strlcat(buf, relay->authlabel, sizeof buf);
-		(void)strlcat(buf, "@", sizeof buf);
+		/* disallow auth on non-tls scheme. */
+		if (relay->tls != RELAY_TLS_STARTTLS &&
+		    relay->tls != RELAY_TLS_SMTPS)
+			return 0;
+		relay->flags |= RELAY_AUTH;
 	}
-	(void)strlcat(buf, relay->hostname, sizeof buf);
-	if (relay->port) {
-		(void)strlcat(buf, ":", sizeof buf);
-		(void)snprintf(port, sizeof port, "%d", relay->port);
-		(void)strlcat(buf, port, sizeof buf);
-	}
-	return buf;
+
+	return 1;
 }
 
 uint64_t
@@ -526,85 +482,94 @@ rule_to_text(struct rule *r)
 	static char buf[4096];
 
 	memset(buf, 0, sizeof buf);
-	(void)strlcpy(buf, r->r_decision == R_ACCEPT  ? "accept" : "reject", sizeof buf);
-	if (r->r_tag[0]) {
-		(void)strlcat(buf, " tagged ", sizeof buf);
-		if (r->r_nottag)
-			(void)strlcat(buf, "! ", sizeof buf);
-		(void)strlcat(buf, r->r_tag, sizeof buf);
-	}
-	(void)strlcat(buf, " from ", sizeof buf);
-	if (r->r_notsources)
-		(void)strlcat(buf, "! ", sizeof buf);
-	(void)strlcat(buf, r->r_sources->t_name, sizeof buf);
-
-	(void)strlcat(buf, " for ", sizeof buf);
-	if (r->r_notdestination)
-		(void)strlcat(buf, "! ", sizeof buf);
-	switch (r->r_desttype) {
-	case DEST_DOM:
-		if (r->r_destination == NULL) {
-			(void)strlcat(buf, " any", sizeof buf);
-			break;
-		}
-		(void)strlcat(buf, " domain ", sizeof buf);
-		(void)strlcat(buf, r->r_destination->t_name, sizeof buf);
-		if (r->r_mapping) {
-			(void)strlcat(buf, " alias ", sizeof buf);
-			(void)strlcat(buf, r->r_mapping->t_name, sizeof buf);
-		}
-		break;
-	case DEST_VDOM:
-		if (r->r_destination == NULL) {
-			(void)strlcat(buf, " any virtual ", sizeof buf);
-			(void)strlcat(buf, r->r_mapping->t_name, sizeof buf);
-			break;
-		}
-		(void)strlcat(buf, " domain ", sizeof buf);
-		(void)strlcat(buf, r->r_destination->t_name, sizeof buf);
-		(void)strlcat(buf, " virtual ", sizeof buf);
-		(void)strlcat(buf, r->r_mapping->t_name, sizeof buf);
-		break;
+	(void)strlcpy(buf, "match", sizeof buf);
+	if (r->flag_tag) {
+		if (r->flag_tag < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " tag ", sizeof buf);
+		(void)strlcat(buf, r->table_tag, sizeof buf);
 	}
 
-	switch (r->r_action) {
-	case A_RELAY:
-		(void)strlcat(buf, " relay", sizeof buf);
-		break;
-	case A_RELAYVIA:
-		(void)strlcat(buf, " relay via ", sizeof buf);
-		(void)strlcat(buf, relayhost_to_text(&r->r_value.relayhost), sizeof buf);
-		break;
-	case A_MAILDIR:
-		(void)strlcat(buf, " deliver to maildir \"", sizeof buf);
-		(void)strlcat(buf, r->r_value.buffer, sizeof buf);
-		(void)strlcat(buf, "\"", sizeof buf);
-		break;
-	case A_MBOX:
-		(void)strlcat(buf, " deliver to mbox", sizeof buf);
-		break;
-	case A_FILENAME:
-		(void)strlcat(buf, " deliver to filename \"", sizeof buf);
-		(void)strlcat(buf, r->r_value.buffer, sizeof buf);
-		(void)strlcat(buf, "\"", sizeof buf);
-		break;
-	case A_MDA:
-		(void)strlcat(buf, " deliver to mda \"", sizeof buf);
-		(void)strlcat(buf, r->r_value.buffer, sizeof buf);
-		(void)strlcat(buf, "\"", sizeof buf);
-		break;
-	case A_LMTP:
-		(void)strlcat(buf, " deliver to lmtp \"", sizeof buf);
-		(void)strlcat(buf, r->r_value.buffer, sizeof buf);
-		(void)strlcat(buf, "\"", sizeof buf);
-		break;
-	case A_NONE:
-		break;
+	if (r->flag_from) {
+		if (r->flag_from < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		if (r->flag_from_socket)
+			(void)strlcat(buf, " from socket", sizeof buf);
+		if (r->flag_from_rdns) {
+			(void)strlcat(buf, " from rdns", sizeof buf);
+			if (r->table_from) {
+				(void)strlcat(buf, " ", sizeof buf);
+				(void)strlcat(buf, r->table_from, sizeof buf);
+			}
+		}
+		else if (strcmp(r->table_from, "<anyhost>") == 0)
+			(void)strlcat(buf, " from any", sizeof buf);
+		else if (strcmp(r->table_from, "<localhost>") == 0)
+			(void)strlcat(buf, " from local", sizeof buf);
+		else {
+			(void)strlcat(buf, " from src ", sizeof buf);
+			(void)strlcat(buf, r->table_from, sizeof buf);
+		}
 	}
 
+	if (r->flag_for) {
+		if (r->flag_for < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		if (strcmp(r->table_for, "<anydestination>") == 0)
+			(void)strlcat(buf, " for any", sizeof buf);
+		else if (strcmp(r->table_for, "<localnames>") == 0)
+			(void)strlcat(buf, " for local", sizeof buf);
+		else {
+			(void)strlcat(buf, " for domain ", sizeof buf);
+			(void)strlcat(buf, r->table_for, sizeof buf);
+		}
+	}
+
+	if (r->flag_smtp_helo) {
+		if (r->flag_smtp_helo < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " helo ", sizeof buf);
+		(void)strlcat(buf, r->table_smtp_helo, sizeof buf);
+	}
+
+	if (r->flag_smtp_auth) {
+		if (r->flag_smtp_auth < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " auth", sizeof buf);
+		if (r->table_smtp_auth) {
+			(void)strlcat(buf, " ", sizeof buf);
+			(void)strlcat(buf, r->table_smtp_auth, sizeof buf);
+		}
+	}
+
+	if (r->flag_smtp_starttls) {
+		if (r->flag_smtp_starttls < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " tls", sizeof buf);
+	}
+
+	if (r->flag_smtp_mail_from) {
+		if (r->flag_smtp_mail_from < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " mail-from ", sizeof buf);
+		(void)strlcat(buf, r->table_smtp_mail_from, sizeof buf);
+	}
+
+	if (r->flag_smtp_rcpt_to) {
+		if (r->flag_smtp_rcpt_to < 0)
+			(void)strlcat(buf, " !", sizeof buf);
+		(void)strlcat(buf, " rcpt-to ", sizeof buf);
+		(void)strlcat(buf, r->table_smtp_rcpt_to, sizeof buf);
+	}
+	(void)strlcat(buf, " action ", sizeof buf);
+	if (r->reject)
+		(void)strlcat(buf, "reject", sizeof buf);
+	else
+		(void)strlcat(buf, r->dispatcher, sizeof buf);
 	return buf;
 }
 
+
 int
 text_to_userinfo(struct userinfo *userinfo, const char *s)
 {
@@ -698,7 +663,6 @@ text_to_expandnode(struct expandnode *expandnode, const char *s)
 	    alias_is_filter(expandnode, s, l) ||
 	    alias_is_filename(expandnode, s, l) ||
 	    alias_is_address(expandnode, s, l) ||
-	    alias_is_maildir(expandnode, s, l) ||
 	    alias_is_username(expandnode, s, l))
 		return (1);
 
@@ -713,8 +677,6 @@ expandnode_to_text(struct expandnode *expandnode)
 	case EXPAND_FILENAME:
 	case EXPAND_INCLUDE:
 	case EXPAND_ERROR:
-	case EXPAND_MAILDIR:
-		return expandnode->u.buffer;
 	case EXPAND_USERNAME:
 		return expandnode->u.user;
 	case EXPAND_ADDRESS:
@@ -726,22 +688,6 @@ expandnode_to_text(struct expandnode *expandnode)
 	return NULL;
 }
 
-static int
-alias_is_maildir(struct expandnode *alias, const char *line, size_t len)
-{
-	if (strncasecmp("maildir:", line, 8) != 0)
-		return (0);
-
-	line += 8;
-	memset(alias, 0, sizeof *alias);
-	alias->type = EXPAND_MAILDIR;
-	if (strlcpy(alias->u.buffer, line,
-	    sizeof(alias->u.buffer)) >= sizeof(alias->u.buffer))
-		return (0);
-
-	return (1);
-}
-
 /******/
 static int
 alias_is_filter(struct expandnode *alias, const char *line, size_t len)
