@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.132 2017/01/09 14:49:22 reyk Exp $	*/
+/*	$OpenBSD: util.c,v 1.150 2019/10/03 04:49:12 gilles Exp $	*/
 
 /*
  * Copyright (c) 2000,2001 Markus Friedl.  All rights reserved.
@@ -47,6 +47,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -61,58 +62,67 @@ int	tracing = 0;
 int	foreground_log = 0;
 
 void *
-xmalloc(size_t size, const char *where)
+xmalloc(size_t size)
 {
 	void	*r;
 
-	if ((r = malloc(size)) == NULL) {
-		log_warnx("%s: malloc(%zu)", where, size);
-		fatalx("exiting");
-	}
+	if ((r = malloc(size)) == NULL)
+		fatal("malloc");
 
 	return (r);
 }
 
 void *
-xcalloc(size_t nmemb, size_t size, const char *where)
+xcalloc(size_t nmemb, size_t size)
 {
 	void	*r;
 
-	if ((r = calloc(nmemb, size)) == NULL) {
-		log_warnx("%s: calloc(%zu, %zu)", where, nmemb, size);
-		fatalx("exiting");
-	}
+	if ((r = calloc(nmemb, size)) == NULL)
+		fatal("calloc");
 
 	return (r);
 }
 
 char *
-xstrdup(const char *str, const char *where)
+xstrdup(const char *str)
 {
 	char	*r;
 
-	if ((r = strdup(str)) == NULL) {
-		log_warnx("%s: strdup(%p)", where, str);
-		fatalx("exiting");
-	}
+	if ((r = strdup(str)) == NULL)
+		fatal("strdup");
 
 	return (r);
 }
 
 void *
-xmemdup(const void *ptr, size_t size, const char *where)
+xmemdup(const void *ptr, size_t size)
 {
 	void	*r;
 
-	if ((r = malloc(size)) == NULL) {
-		log_warnx("%s: malloc(%zu)", where, size);
-		fatalx("exiting");
-	}
+	if ((r = malloc(size)) == NULL)
+		fatal("malloc");
+
 	memmove(r, ptr, size);
 
 	return (r);
 }
 
+int
+xasprintf(char **ret, const char *format, ...)
+{
+	int r;
+	va_list ap;
+
+	va_start(ap, format);
+	r = vasprintf(ret, format, ap);
+	va_end(ap);
+	if (r == -1)
+		fatal("vasprintf");
+
+	return (r);
+}
+
+
 #if !defined(NO_IO)
 int
 io_xprintf(struct io *io, const char *fmt, ...)
@@ -168,72 +178,13 @@ bsnprintf(char *str, size_t size, const char *format, ...)
 	va_start(ap, format);
 	ret = vsnprintf(str, size, format, ap);
 	va_end(ap);
-	if (ret == -1 || ret >= (int)size)
+	if (ret < 0 || ret >= (int)size)
 		return 0;
 
 	return 1;
 }
 
 
-static int
-mkdirs_component(char *path, mode_t mode)
-{
-	struct stat	sb;
-
-	if (stat(path, &sb) == -1) {
-		if (errno != ENOENT)
-			return 0;
-		if (mkdir(path, mode | S_IWUSR | S_IXUSR) == -1)
-			return 0;
-	}
-	else if (!S_ISDIR(sb.st_mode))
-		return 0;
-
-	return 1;
-}
-
-int
-mkdirs(char *path, mode_t mode)
-{
-	char	 buf[PATH_MAX];
-	int	 i = 0;
-	int	 done = 0;
-	char	*p;
-
-	/* absolute path required */
-	if (*path != '/')
-		return 0;
-
-	/* make sure we don't exceed PATH_MAX */
-	if (strlen(path) >= sizeof buf)
-		return 0;
-
-	memset(buf, 0, sizeof buf);
-	for (p = path; *p; p++) {
-		if (*p == '/') {
-			if (buf[0] != '\0')
-				if (!mkdirs_component(buf, mode))
-					return 0;
-			while (*p == '/')
-				p++;
-			buf[i++] = '/';
-			buf[i++] = *p;
-			if (*p == '\0' && ++done)
-				break;
-			continue;
-		}
-		buf[i++] = *p;
-	}
-	if (!done)
-		if (!mkdirs_component(buf, mode))
-			return 0;
-
-	if (chmod(path, mode) == -1)
-		return 0;
-
-	return 1;
-}
-
 int
 ckdir(const char *path, mode_t mode, uid_t owner, gid_t group, int create)
 {
@@ -457,7 +408,7 @@ mailaddr_match(const struct mailaddr *maddr1, const struct mailaddr *maddr2)
 	if (m2.user[0] == '\0' && m2.domain[0] == '\0')
 		return 1;
 
-	if (!hostname_match(m1.domain, m2.domain))
+	if (m2.domain[0] && !hostname_match(m1.domain, m2.domain))
 		return 0;
 
 	if (m2.user[0]) {
@@ -502,6 +453,7 @@ valid_domainpart(const char *s)
 	struct in6_addr	 ina6;
 	char		*c, domain[SMTPD_MAXDOMAINPARTSIZE];
 	const char	*p;
+	size_t		 dlen;
 
 	if (*s == '[') {
 		if (strncasecmp("[IPv6:", s, 6) == 0)
@@ -529,9 +481,71 @@ valid_domainpart(const char *s)
 	if (*s == '\0')
 		return 0;
 
+	dlen = strlen(s);
+	if (dlen >= sizeof domain)
+		return 0;
+
+	if (s[dlen - 1] == '.')
+		return 0;
+
 	return res_hnok(s);
 }
 
+#define LABELCHR(c) ((c) == '-' || (c) == '_' || isalpha((int)(c)) || isdigit((int)(c)))
+#define LABELMAX 63
+#define DNAMEMAX 253
+
+int
+valid_domainname(const char *str)
+{
+	const char *label, *s;
+
+	/*
+	 * Expect a sequence of dot-separated labels, possibly with a trailing
+	 * dot. The empty string is rejected, as well a single dot.
+	 */
+	for (s = str; *s; s++) {
+
+		/* Start of a new label. */
+		label = s;
+		while (LABELCHR(*s))
+			s++;
+
+		/* Must have at least one char and at most LABELMAX. */
+		if (s == label || s - label > LABELMAX)
+			return 0;
+
+		/* If last label, stop here. */
+		if (*s == '\0')
+			break;
+
+		/* Expect a dot as label separator or last char. */
+		if (*s != '.')
+			return 0;
+	}
+
+	/* Must have at leat one label and no more than DNAMEMAX chars. */
+	if (s == str || s - str > DNAMEMAX)
+		return 0;
+
+	return 1;
+}
+
+int
+valid_smtp_response(const char *s)
+{
+	if (strlen(s) < 5)
+		return 0;
+
+	if ((s[0] < '2' || s[0] > '5') ||
+	    (s[1] < '0' || s[1] > '9') ||
+	    (s[2] < '0' || s[2] > '9') ||
+	    (s[3] != ' '))
+		return 0;
+
+	return 1;
+}
+
 int
 secure_file(int fd, char *path, char *userdir, uid_t uid, int mayread)
 {
@@ -547,7 +561,7 @@ secure_file(int fd, char *path, char *userdir, uid_t uid, int mayread)
 		homedir[0] = '\0';
 
 	/* Check the open file to avoid races. */
-	if (fstat(fd, &st) < 0 ||
+	if (fstat(fd, &st) == -1 ||
 	    !S_ISREG(st.st_mode) ||
 	    st.st_uid != uid ||
 	    (st.st_mode & (mayread ? 022 : 066)) != 0)
@@ -559,7 +573,7 @@ secure_file(int fd, char *path, char *userdir, uid_t uid, int mayread)
 			return 0;
 		(void)strlcpy(buf, cp, sizeof(buf));
 
-		if (stat(buf, &st) < 0 ||
+		if (stat(buf, &st) == -1 ||
 		    (st.st_uid != 0 && st.st_uid != uid) ||
 		    (st.st_mode & 022) != 0)
 			return 0;
@@ -687,8 +701,6 @@ session_socket_error(int fd)
 const char *
 parse_smtp_response(char *line, size_t len, char **msg, int *cont)
 {
-	size_t	 i;
-
 	if (len >= LINE_MAX)
 		return "line too long";
 
@@ -710,11 +722,6 @@ parse_smtp_response(char *line, size_t len, char **msg, int *cont)
 	    !isdigit((unsigned char)line[2]))
 		return "reply code out of range";
 
-	/* validate reply message */
-	for (i = 0; i < len; i++)
-		if (!isprint((unsigned char)line[i]))
-			return "non-printable character in reply";
-
 	return NULL;
 }
 
@@ -776,6 +783,7 @@ getmailname(char *hostname, size_t len)
 
 	if (strlcpy(hostname, res->ai_canonname, len) >= len) {
 		fprintf(stderr, "hostname too long");
+		freeaddrinfo(res);
 		return -1;
 	}
 
@@ -796,6 +804,26 @@ base64_decode(char const *src, unsigned char *dest, size_t destsize)
 	return __b64_pton(src, dest, destsize);
 }
 
+int
+base64_encode_rfc3548(unsigned char const *src, size_t srclen,
+	      char *dest, size_t destsize)
+{
+	size_t i;
+	int ret;
+
+	if ((ret = base64_encode(src, srclen, dest, destsize)) == -1)
+		return -1;
+
+	for (i = 0; i < destsize; ++i) {
+		if (dest[i] == '/')
+			dest[i] = '_';
+		else if (dest[i] == '+')
+			dest[i] = '-';
+	}
+
+	return ret;
+}
+
 void
 log_trace(int mask, const char *emsg, ...)
 {
@@ -827,3 +855,16 @@ xclosefrom(int lowfd)
     closefrom(lowfd);
 #endif
 }
+
+void
+portable_freeaddrinfo(struct addrinfo *ai)
+{
+	struct addrinfo *p;
+
+	do {
+		p = ai;
+		ai = ai->ai_next;
+		free(p->ai_canonname);
+		free(p);
+	} while (ai);
+}
